#!/usr/sbin/perl 
#
# rep_aln.pl
#
# Version 1.1: swise 1.7 fails when working with several sequences
#              (at least my binary running on dove/booby). I make
#              the script to work separately for each sequence.
# Miguel - 30 May 2002
#
# Miguel Andrade - 28 May 2002
# Repeats project
#===============================================


# constants
$TMP_DIR = "tmp/";
$DATA_DIR = "data/";
$repeater_script = "/booby1/home/andrade/public_html/rep/repeater.perl";
$select_script = "/booby1/home/andrade/public_html/rep/select.perl";

# correspondences between repeat name and the *.prf *.for file names

%basic_name = (
               'ANK', 'ank2',
               'ARM', 'arm',
               'HAT', 'hat',
               'HEAT', 'heat',
	       'HEAT_AAA', 'heat_aaa',
	       'HEAT_ADB', 'heat_adb',
	       'HEAT_IMB', 'heat_imb',
               'KELCH', 'kelch',
               'LRR', 'LRR_typ',
               'PFTA', 'pfta',
               'PFTB', 'pftb3',
               'RCC1', 'rcc1',
               'TPR', 'tpr',
               'WD40', 'wd3',
);

%e_value = (
               'ANK', 1e-5,
               'ARM', 1e-8,
               'HAT', 1e-4,
               'HEAT', 1e-6,
	       'HEAT_AAA', 1e-5,
	       'HEAT_ADB', 1e-6,
	       'HEAT_IMB', 1e-6,
               'KELCH', 1e-4,
               'LRR', 1e-4,
               'PFTA', 1e-4,
               'PFTB', 1e-5,
               'RCC1', 1e-5,
               'TPR', 1e-4,
               'WD40', 1e-4,
);

%min_rep = (
               'ANK', 1,
               'ARM', 3,
               'HAT', 3,
               'HEAT', 4,
	       'HEAT_AAA', 5,
	       'HEAT_ADB', 4,
	       'HEAT_IMB', 5,
               'KELCH', 3,
               'LRR', 2,
               'PFTA', 4,
               'PFTB', 2,
               'RCC1', 3,
               'TPR', 3,
               'WD40', 3,
);


require "./cgi-lib.pl";

# Start off by reading and parsing the data.  Save the return value.
$ret = &ReadParse;

$alignment = $in{'alignment'};
$repeat = $in{'repeat'};

if ($repeat eq ''){
  $error = "You must select a repeat family!";
  &sub_error;
  exit;
}

if ($alignment eq ''){
  $error = "You must input an alignment!";
  &sub_error;
  exit;
}


# split alignment
@aln = split(/\n/, $alignment);


# read the clustalw alignment

&html_header;

$aln_ok = 1;
$header = 1;
foreach $line(@aln){
  chop($line);
  if ($header > 0){
    $header--;
    next;
  }
  if ($line =~ /^\S/){
    $line_length = length($line);
    ($seq_id, $frg) = split(/\s+/, $line);
    $seq{$seq_id} .= $frg;
    
    # check for the separation between sequence fragment and label
    $this_label_room = $line_length - length($frg);
    unless (exists $label_room{$seq_id}){
      $label_room{$seq_id} = $this_label_room;
    } else {
      # check for coherence
      if ($label_room{$seq_id} != $this_label_room){
        $aln_ok = 0;
      }
    }  
  }
}


if ($aln_ok == 0){
  $error = "Problem with the alignment format";
  &sub_error;
  exit;
}

$TIME_STAMP = time;
$FASTA = $TMP_DIR.$TIME_STAMP.".fasta";

# prepare commands for repeater and select

$PAR = $TMP_DIR.$TIME_STAMP.".par";
$neg_profile = $DATA_DIR.$basic_name{$repeat}."_neg.prf";
$pos_profile = $DATA_DIR.$basic_name{$repeat}."_pos.prf";
$for_file = $DATA_DIR.$basic_name{$repeat}."_neg.for";
$par_file = $TMP_DIR.$internal_id.".par";
$sel_file = $TMP_DIR.$internal_id.".sel";
$command1 = "$repeater_script -neg $neg_profile -pos $pos_profile -d $FASTA -mode search -for $for_file -thr 1e-2 > $PAR";

$SEL = $TMP_DIR.$TIME_STAMP.".sel";
$command2 = "$select_script $PAR $e_value{$repeat} $e_value{$repeat} $min_rep{$repeat} 10000 > $SEL";


# v 1.1 to solve problems with swise and multiple alignments
# we make the analysis separately for each sequence

foreach $seq_id(keys %seq){
  # clean sequence and make a fasta file
  open(FASTA,">".$FASTA);
  $seq{$seq_id} =~ s/\-//g;
  print FASTA ">$seq_id\n";
  print FASTA "$seq{$seq_id}\n";
  close(FASTA);

  # call repeater on the fasta file with the selected repeat
  `$command1`;

  # call select on the par file with the appropriate thresholds
  `$command2`;

  # parse the select file in order to get the repeats
  open(SEL,$SEL);
  $header = 3;

  while(<SEL>){
    if ($header > 0){
      $header--;
      next;
    }
    @words = split(/\t/);

    # note that the repeat id contains the start position
    # we can do this because we know that there are no overlapping
    # repeats.
    $id = $words[0];
    $rep_id = $id." ".$words[6];
    $start{$rep_id} = 1;

    $score{$rep_id} = $words[5];
    $id{$rep_id} = $words[0];
    $order{$rep_id} = $words[1];

    # annotate rank of scores
    if (($score{$rep_id} < $min_score) or ($min_score eq '')){
      $min_score = $score{$rep_id};
    }
    if (($score{$rep_id} > $max_score) or ($max_score eq '')){
      $max_score = $score{$rep_id};
    }

    # the rep id for the stop contains the stop position
    $rep_id2 = $id." ".$words[7];
    $stop{$rep_id2} = 1;

  }
  close(SEL);
}


$score_thr1 = $min_score + ($max_score - $min_score)/2;

# put back the alignment with colour coding and rep consensus
$header = 3;
$prev_blank = 0;
$num_seq = 0;


print "<H1>Scan of an alignment for $repeat repeats</H1>\n";

if ($min_score eq ''){
  print "Sorry! No repeats identified above the threshold used.\n";
} else {
  print "<B>Colour code in the alignment</B><BR>\n";
  print "<FONT COLOR=green>Green:</FONT> start of repeat<BR>\n";
  print "<FONT COLOR=red>Red:</FONT> end of repeat<BR>\n";
  print "<FONT COLOR=blue>Blue:</FONT> repeat with score between $min_score and $score_thr1<BR>\n";
  print "<FONT COLOR=cyan>Cyan:</FONT> repeat with score between $score_thr1 and $max_score<BR>\n";
  print "<P>\n";

  print "<B>Consensus line at 50%</B><BR>\n";
  print "<B>[</B> start of repeat for half or more of the aligned sequences<BR>\n";
  print "<B>]</B> end of repeat for half or more of the aligned sequences<BR>\n";
  print "<B>R</B> repeat position for half or more of the aligned sequences<BR>\n";
  print "<P>\n";
}

print "<PRE>\n";

foreach $line(@aln){
  if ($header > 0){
    print "$line\n";
    $header--;
    next;
  }
  if ($line =~ /^\S/){

    $num_seq++;
    ($seq_id, $frg) = split(/\s+/, $line);
    print $seq_id;
    for ($j = 0; $j < $label_room{$seq_id} - length($seq_id); $j++){
      print " ";
    }

    for ($column = 0; $column < length($frg); $column++){
      $char = substr($frg, $column, 1);
      if ($char ne '-'){
        $position{$seq_id}++;
        if ($rep_id{$seq_id} eq ''){
          # we were not inside a predicted repeat

          # check for the start of a repeat
          $rep_id{$seq_id} = $seq_id." ".$position{$seq_id};
          if (exists $start{$rep_id{$seq_id}}){
            $consensus_start{$column}++;
            $consensus_rep{$column}++;
            $score_rep{$seq_id} = $score{$rep_id{$seq_id}};
            print "<FONT COLOR=GREEN>$char</FONT>";
          } else {
            # no start of repeat
            print $char;
	    $rep_id{$seq_id} = '';
          }
        } else {
          # we were inside a predicted repeat

          # check for the end of a repeat
          $rep_id{$seq_id} = $seq_id." ".$position{$seq_id};
          if (exists $stop{$rep_id{$seq_id}}){
            $consensus_stop{$column}++;
            $consensus_rep{$column}++;
            print "<FONT COLOR=RED>$char</FONT>";
            $rep_id{$seq_id} = '';
          } else {
            # the repeat goes on
            $consensus_rep{$column}++;
            if ($score_rep{$seq_id} > $score_thr1){
              print "<FONT COLOR=CYAN>$char</FONT>";
            } else {
              print "<FONT COLOR=BLUE>$char</FONT>";
            }
          }
        }

      } else {
        # print the gap
        print $char;
      }
    }
    # close the line
    print "\n";


    $prev_blank = 0;

  } else {
    if ($prev_blank == 0){
      # print consensus line
      
      $half_seq = $num_seq/2;

      $rep_label = "REP cons 50%";
      print $rep_label;
      for ($j = 0; $j < $this_label_room - length($rep_label); $j++){
	print " ";
      }
      
      for ($column = 0; $column < length($frg); $column++){
        if ($consensus_start{$column} > $half_seq){
          print "[";
        } elsif ($consensus_stop{$column} > $half_seq){
          print "]";
        } elsif ($consensus_rep{$column} > $half_seq){
          print "R";
        } else {
          print ".";
        }
      }
      print "\n";
      undef %consensus_stop;
      undef %consensus_start;
      undef %consensus_rep;
    }

    print "$line\n";
    $prev_blank = 1;
    $num_seq = 0.
  }
}
print "</PRE>\n";


# delete the fasta file
`/usr/bin/rm $FASTA`;

# delete the par file
`/usr/bin/rm $PAR`;

# delete the sel file
`/usr/bin/rm $SEL`;



sub sub_error
  {
    &html_header;
    print "<H3>Error: $error</H3>\n";
    &html_bottom;
  }

sub html_header
  {
    print "Content-type: text/html\n\n";
    print "<BODY BGCOLOR=#FFFFFF TEXT=#000000 LINK=#808800>\n";
  }

sub html_bottom
  {
    print "<HR>\n";
    print "<A HREF=http://www.embl-heidelberg.de/~andrade/papers/rep/>[REP Home]</A>\n";
    print "<A HREF=http://www.embl-heidelberg.de/~andrade/index.html>[Andrade Home]</A>\n";
    print "<A HREF=http://www.bork.embl-heidelberg.de/>[Bork Group]</A>\n";
  }

