#!/usr/pub/bin/perl5.00404 



%basic_name = (
               'ANK', 'ank2',
               'ARM', 'arm',
               'HAT', 'hat',
               'HEAT', 'heat',
               'KELCH', 'kelch',
               'LRR', 'lrr2',
               'PFTA', 'pfta',
               'PFTB', 'pftb3',
               'RCC1', 'rcc1',
               'TPR', 'tpr',
               'WD40', 'wd3',
);

%e_value = (
               'ANK', 1e-5,
               'ARM', 1e-8,
               'HAT', 1e-4,
               'HEAT', 1e-6,
               'KELCH', 1e-4,
               'LRR', 1e-4,
               'PFTA', 1e-4,
               'PFTB', 1e-5,
               'RCC1', 1e-5,
               'TPR', 1e-4,
               'WD40', 1e-4,
);

%min_rep = (
               'ANK', 1,
               'ARM', 3,
               'HAT', 3,
               'HEAT', 4,
               'KELCH', 3,
               'LRR', 2,
               'PFTA', 4,
               'PFTB', 2,
               'RCC1', 3,
               'TPR', 3,
               'WD40', 3,
);


require "./cgi-lib.pl";

$row = '/biocomp/bork1/bin/row';
$column = '/biocomp/bork1/bin/column';

# Start off by reading and parsing the data.  Save the return value.
# We could also save the file's name and content type, but we don't
# do that in this example.
$ret = &ReadParse;

$sequence = $in{'sequence'};
$search_id = $in{'search_id'};

# clean the sequence
$sequence =~ tr/[a-z]/[A-Z]/;
$sequence =~ s/\W//g;
$sequence =~ s/\d//g;

if ($sequence eq '') {
  $error = "You have to provide a protein sequence";
  &sub_error;
  exit;
} elsif ($search_id eq '') {
  $error = "You have to provide an id for your search";
  &sub_error;
  exit;
}

$length = length($sequence);
if ($length > 10000) {
  $error = "Sequence is too long (> 10 000 characters). Submit fragments.";
  &sub_error;
  exit;
} elsif ($length < 20) {
  $error = "Sequence is too short (< 20 characteres). I will not find a repeat there.";
  &sub_error;
  exit;
}

if ($search_id =~ /\W/) {
  $error = "Search identifier must be one word composed of only letters and/or numbers";
  &sub_error;
  exit;
}
if (length($search_id) > 20) {
  $error = "Search identifier should be shorter than 20 characters";
  &sub_error;
  exit;
}


$time = time;
$internal_id = $search_id.'_'.$time;

# make the sequence in a nice 60 column format
$i = 0;
$j = 0;
while ($i < (length($sequence) - 60)) {
  $nice_seq[$j++] = substr($sequence, $i, 60);
  $i += 60;
}
$nice_seq[$j++] = substr($sequence, $i, length($sequence));

$fasta_file = 'tmp/'.$internal_id.'.fasta';

open(FASTA, '>'.$fasta_file);
print FASTA ">$search_id\n";
foreach $line(@nice_seq){
  print FASTA "$line\n";
}
close(FASTA);



# call repeater and select for each of the repeats

$rep = 'PFTA';
$neg_profile = 'data/'.$basic_name{$rep}.'_neg.prf';
$pos_profile = 'data/'.$basic_name{$rep}.'_pos.prf';
$for_file = 'data/'.$basic_name{$rep}.'_neg.for';
$par_file = 'tmp/'.$internal_id.'.par';
$sel_file = 'tmp/'.$internal_id.'.sel';



$repeater_call = "/booby1/home/andrade/public_html/rep/repeater.perl -neg $neg_profile -pos $pos_profile -d $fasta_file -mode search -for $for_file -thr 1e-2 > $par_file";

`$repeater_call`;

$select_call = "/booby1/home/andrade/public_html/rep/select.perl $par_file $e_value{$rep} $e_value{$rep} $min_rep{$rep} 10000";

@select_lines = `$select_call`; 

$counter = 2;
foreach $line(@select_lines){
  if ($line =~ /^\#/) {
    next;
  }
  if (counter > 0) {
    $counter--;
  }

  chop;
  @words = split(/\t/, $line);
  $evalue = $words[3];
  $score = $words[5];
  $from = $words[6];
  $to = $words[7];

  $repeat_id = $rep.':'.$from.':'.$to.':'.$evalue.':'.$score;
  $repeat_list{$repeat_id} = 1;
  
}









# output

&html_header;
print "<TITLE>Results of $search_id search</TITLE>\n";
print "<H1>Results of $search_id search</H1>\n";

print "Submitted sequence:<BR>\n";
print "<PRE>\n";
foreach $line(@nice_seq){
  print "$line\n";
}
print "</PRE>\n";


foreach $properties(keys %repeat_list){
  print "$properties<BR>";
}


&html_bottom;





sub sub_error
  {
    &html_header;
    print "<H3>Error: $error</H3>\n";
    &html_bottom;
  }

sub html_header
  {
    print "Content-type: text/html\n\n";
    print "<BODY BGCOLOR=#FFFFFF TEXT=#008800 LINK=#808800>\n";
  }

sub html_bottom
  {
    print "<HR>\n";
    print "<A HREF=http://www.embl-heidelberg.de/~andrade/papers/rep/>[Appendix Home]</A>\n";
    print "<A HREF=http://www.embl-heidelberg.de/~andrade/index.html>[Andrade Home]</A>\n";
    print "<A HREF=http://www.bork.embl-heidelberg.de/>[Bork Group]</A>\n";
  }
